#include <stdio.h>
#include <sctp.h>

int 
SCTP_initLibrary(void)
{
    int result;
    
    if ((result = sctp_initLibrary()) != 0) {
        if (result == -1) {
            fprintf(stderr, "sctp_initLibrary: called muliple times.\n");
        } else 
        if (result == 1) {
            fprintf(stderr, "sctp_initLibrary: could not open raw socket for SCTP.\n");
        } else {
            fprintf(stderr, "sctp_initLibrary: unknown value (%i) returned.\n", result);
        }
        fflush(stderr);
    }
    return result;
};

unsigned short
SCTP_registerInstance(unsigned short port,
                      unsigned short noOfInStreams,
                      unsigned short noOfOutStreams,
                      unsigned int noOfLocalAddresses,
                      unsigned char localAddressList[][SCTP_MAX_IP_LEN],
                      SCTP_ulpCallbacks ULPcallbackFunctions)
{
    unsigned short result;
    
    if ((result = sctp_registerInstance(port, 
                                       noOfInStreams, noOfOutStreams,
                                       noOfLocalAddresses, localAddressList,
                                       ULPcallbackFunctions)) != 0) {
        fprintf(stderr, "sctp_registerInstance: an error occured.\n");
        fflush(stderr);        
    }
    return result;
};

int
SCTP_deleteAssociation(unsigned int associationID)
{
    int result;
    
    if ((result = sctp_deleteAssociation(associationID))!=0) {
        if (result == -1) {
            fprintf(stderr, "sctp_deleteAssociation: assoc not ready for deletion or lib not initialized:\n");
        } else
        if (result == 1) {
            fprintf(stderr, "sctp_deleteAssociation: assoc does not exists.\n");
        } else {
            fprintf(stderr, "sctp_deleteAssociation: unkown value (%i) returned.\n", result);
        }
        fflush(stderr);
    }
    return result;
};

int 
SCTP_send(unsigned int associationID, unsigned short streamID,
          unsigned char *buffer, unsigned int length, unsigned int protocolId, short path_id,
          void*  context, 
          unsigned int lifetime,
          int unorderedDelivery, 
          int dontBundle)     
{
    int result;
    
    if ((result = sctp_send(associationID, streamID,
                           buffer, length, protocolId, path_id,
                           context, 
                           lifetime,
                           unorderedDelivery, 
                           dontBundle))!=0) {
        if (result == 1) {
            fprintf(stderr, "sctp_send: association not found or library not initialized.\n");
        } else
        if (result == -1) {
            fprintf(stderr, "sctp_send: send error.\n");
        } else {
            fprintf(stderr, "sctp_send: unkown result (%i) returned.\n", result);
        }
        fflush(stderr);
    }
    return result;
};

int
SCTP_receive(unsigned int associationID, unsigned short streamID, unsigned char *buffer, 
			unsigned int *length, unsigned int flags)
{
    int result;
    
    if ((result = sctp_receive(associationID, streamID, buffer, length, flags))!=0) {
        if (result == -1) {
            fprintf(stderr, "sctp_receive: association not found.\n");
        } else 
        if (result == 1) {
            fprintf(stderr, "sctp_receive: no data.\n");
        } else {
            fprintf(stderr, "sctp_receive: unknown value (%i) returned.\n", result);
        }
        fflush(stderr);
    }
    return result;
};

int SCTP_shutdown(unsigned int associationID)
{
    int result;
    
    if ((result = sctp_shutdown(associationID))!=0) {
        if (result == -1) {
            fprintf(stderr, "sctp_shutdown: library not initialized.\n");
        } else
        if (result == 1) {
            fprintf(stderr, "sctp_shutdown: association not found.\n");
        } else {
            fprintf(stderr, "sctp_shutdown: unknown value (%i) returned.\n", result);
        }
        fflush(stderr);
    }
    return result;
};

int
SCTP_eventLoop(void)
{
    int result;
    
    if ((result = sctp_eventLoop()) < 0) {
        if (result == -1) {
            fprintf(stderr, "sctp_eventLoop: an error occured.\n");
        } else {
            fprintf(stderr, "sctp_eventLoop: unknown value (%i) returned.\n", result);
        }
        fflush(stderr);
    }
    return result;
};
