


#include "breakdetector.h"
#include "tools.h"


#include <signal.h>


//namespace Coral {


// Kill after timeout: Send kill signal, if Ctrl-C is pressed again
// after more than KILL_TIMEOUT microseconds,
#define KILL_AFTER_TIMEOUT
#define KILL_TIMEOUT 2000000



// ###### Global variables ##################################################
bool   DetectedBreak = false;
bool   PrintedBreak  = false;
#ifdef KILL_AFTER_TIMEOUT
bool   PrintedKill   = false;
card64 LastDetection = (card64)-1;
#endif


// ###### Handler for SIGINT ################################################
void breakDetector(int signum)
{
   DetectedBreak = true;

#ifdef KILL_AFTER_TIMEOUT
   if(!PrintedKill) {
      const card64 now = getMicroTime();
      if(LastDetection == (card64)-1) {
         LastDetection = now;
      }
      else if(now - LastDetection >= 2000000) {
         PrintedKill = true;
         cerr << endl << "*** Kill ***" << endl << endl;
         kill(getpid(),SIGKILL);
      }
   }
#endif
}


// ###### Install break detector ############################################
void installBreakDetector()
{
   DetectedBreak = false;
   PrintedBreak  = false;
#ifdef KILL_AFTER_TIMEOUT
   PrintedKill   = false;
   LastDetection = (card64)-1;
#endif
   signal(SIGINT,&breakDetector);
}


// ###### Unnstall break detector ###########################################
void uninstallBreakDetector()
{
   signal(SIGINT,SIG_DFL);
#ifdef KILL_AFTER_TIMEOUT
   PrintedKill   = false;
   LastDetection = (card64)-1;
#endif
   DetectedBreak = false;
   PrintedBreak  = false;
}


// ###### Check, if break has been detected #################################
bool breakDetected()
{
   if((DetectedBreak) && (!PrintedBreak)) {
      cerr << endl << "*** Break ***    Signal #" << SIGINT << endl << endl;
      PrintedBreak = getMicroTime();
   }
   return(DetectedBreak);
}


// }
