


#include "condition.h"


#include <sys/time.h>


// namespace Coral {


// ###### Constructor #######################################################
Condition::Condition()
{
   pthread_cond_init(&ConditionVariable,NULL);
}


// ###### Destructor ########################################################
Condition::~Condition()
{
   pthread_cond_destroy(&ConditionVariable);
}


// ###### Wait for condition with timeout ###################################
bool Condition::timedWait(const card64 microseconds)
{
   synchronized();

   // ====== Initialize timeout settings ====================================
   timeval  now;
   timespec timeout;
   gettimeofday(&now,NULL);
   timeout.tv_sec  = now.tv_sec + (long)(microseconds / 1000000);
   timeout.tv_nsec = (now.tv_usec + (long)(microseconds % 1000000)) * 1000;
   if(timeout.tv_nsec >= 1000000000) {
      timeout.tv_sec++;
      timeout.tv_nsec -= 1000000000;
   }

   // ====== Wait ===========================================================
   int result = pthread_cond_timedwait(&ConditionVariable,&Mutex,&timeout);
   while(result == EINTR) {
      result = pthread_cond_timedwait(&ConditionVariable,&Mutex,&timeout);
   }

   unsynchronized();
   return(result == 0);
}


//}
