

#include "sctpassociation.h"
#include "sctpsocketmaster.h"


// namespace Coral {


#define PRINT_SHUTDOWN


// ###### Constructor #######################################################
SCTPAssociation::SCTPAssociation(SCTPSocket*        socket,
                                 const unsigned int associationID)
{
   Socket                        = socket;
   AssociationID                 = associationID;
   CommunicationLostNotification = false;
   ShutdownCompleteNotification  = false;
   IsShuttingDown                = false;
   Socket->SCTPAssociationList.insert(pair<unsigned int, SCTPAssociation*>(AssociationID,this));
}


// ###### Destructor ########################################################
SCTPAssociation::~SCTPAssociation()
{
   SCTPSocketMaster::MasterInstance.lock();
   if(!ShutdownCompleteNotification)
   {
#ifdef PRINT_SHUTDOWN
      cout << "Active shutdown of association #" << AssociationID << " started..." << endl;
#endif
      shutdown();
      SCTPSocketMaster::MasterInstance.unlock();
      if(ShutdownCompleteCondition.timedWait(5000000) == false) {
#ifdef PRINT_SHUTDOWN
         cout << "Active shutdown of association #" << AssociationID << " failed -> sending abort!" << endl;
#endif
         abort();
      }
#ifdef PRINT_SHUTDOWN
      cout << "Active shutdown of association #" << AssociationID << " complete!" << endl;
#endif
   }
   else {
#ifdef PRINT_SHUTDOWN
      cout << "Passive shutdown of association #" << AssociationID << "!" << endl;
#endif
      SCTPSocketMaster::MasterInstance.unlock();
   }

   SCTPSocketMaster::MasterInstance.lock();
   sctp_deleteAssociation(AssociationID);
   multimap<unsigned int, SCTPAssociation*>::iterator iterator =
      Socket->SCTPAssociationList.find(AssociationID);
   if(iterator != Socket->SCTPAssociationList.end()) {
      Socket->SCTPAssociationList.erase(iterator);
   }
   SCTPSocketMaster::MasterInstance.unlock();
}


// ###### Get association ID ################################################
unsigned int SCTPAssociation::getID() const
{
   return(AssociationID);
}


// ###### Get local address #################################################
bool SCTPAssociation::getLocalAddresses(InternetAddress** addressArray,
                                        unsigned int&     addressEntries)
{
   return(Socket->getLocalAddresses(addressArray,addressEntries));
}


// ###### Get remote address ################################################
bool SCTPAssociation::getRemoteAddresses(InternetAddress** addressArray,
                                         unsigned int&     addressEntries)
{
   bool                    result = true;
   SCTP_Association_Status status;

   SCTPSocketMaster::MasterInstance.lock();

   if(sctp_getAssocStatus(AssociationID,&status) == 0) {
      unsigned int i;
      for(i = 0;i < status.numberOfAddresses;i++) {
         SCTP_Path_Status pathStatus;
         if(sctp_getPathStatus(AssociationID,i,&pathStatus) != 0) {
            cerr << "INTERNAL ERROR: SCTPSocketMaster::getRemoteAddress() - sctp_getPathStatus() failure!"
                 << endl;
            exit(1);
         }
         if(i >= addressEntries) {
            result = false;
            break;
         }
         addressArray[i]->init((char*)&pathStatus.destinationAddress,status.destPort);
      }
      addressEntries = i;
   }

   SCTPSocketMaster::MasterInstance.unlock();
   return(result);
}


// ###### Receive ###########################################################
int SCTPAssociation::receive(char*           dataBuffer,
                             size_t&         dataBufferSize,
                             char*           controlBuffer,
                             size_t&         controlBufferSize,
                             int&            flags,
                             unsigned short& streamID,
                             unsigned int&   protoID)
{
   unsigned int assocID = AssociationID;
   streamID             = (unsigned short)-1;
   AssociationStatus status;
   return(Socket->internalReceive(
            InQueue,
            dataBuffer, dataBufferSize,
            controlBuffer, controlBufferSize,
            flags,
            assocID, streamID, protoID,
            status));
}


// ###### Send ##############################################################
int SCTPAssociation::send(const char*          buffer,
                          const size_t         length,
                          const int            flags,
                          const unsigned short streamID,
                          const unsigned int   protoID)
{
   return(Socket->internalSend(
             buffer, length,
             flags,
             AssociationID, streamID, protoID));
}


// ###### Shutdown ##########################################################
void SCTPAssociation::shutdown()
{
   SCTPSocketMaster::MasterInstance.lock();
   if(!IsShuttingDown) {
      IsShuttingDown = true;
      sctp_shutdown(AssociationID);
   }
   SCTPSocketMaster::MasterInstance.unlock();
}


// ###### Abort #############################################################
void SCTPAssociation::abort()
{
   SCTPSocketMaster::MasterInstance.lock();
   IsShuttingDown = true;
   sctp_abort(AssociationID);
   SCTPSocketMaster::MasterInstance.unlock();
}


// }
