

#include "sctpchunkarrivalqueue.h"



// namespace Coral {


// ###### Constructor #######################################################
ChunkArrivalQueue::ChunkArrivalQueue()
{
   FirstChunk = NULL;
   LastChunk  = NULL;
}


// ###### Destructor ########################################################
ChunkArrivalQueue::~ChunkArrivalQueue()
{
   flush();
}


// ###### Add chunk to end of queue ########################################
bool ChunkArrivalQueue::addChunk(const unsigned int       assocID,
                                 const unsigned short     streamID,
                                 const unsigned int       protoID,
                                 const unsigned int       length,
                                 const unsigned int       unordered,
                                 AssociationControlData*  controlData,
                                 const unsigned int       controlDataLength,
                                 const AssociationStatus& status)
{
   Chunk* chunk = new Chunk;
   if(chunk != NULL) {
      chunk->NextChunk         = NULL;
      chunk->AssociationID     = assocID;
      chunk->StreamID          = streamID;
      chunk->Length            = length;
      chunk->ProtoID           = protoID;
      chunk->Unordered         = unordered;
      chunk->Status            = status;
      chunk->ControlData       = controlData;
      chunk->ControlDataLength = controlDataLength;

      if(LastChunk != NULL) {
         LastChunk->NextChunk = chunk;
      }
      LastChunk = chunk;
      if(FirstChunk == NULL) {
         FirstChunk = chunk;
      }

      signal();
      return(true);
   }

   cerr << "ERROR: ChunkArrivalQueue::addChunk() - Out of memory!" << endl;
   return(false);
}


// ###### Add chunk to top of queue ########################################
bool ChunkArrivalQueue::pushChunk(const unsigned int       assocID,
                                  const unsigned short     streamID,
                                  const unsigned int       protoID,
                                  const unsigned int       length,
                                  const unsigned int       unordered,
                                  AssociationControlData*  controlData,
                                  const unsigned int       controlDataLength,
                                  const AssociationStatus& status)
{
   Chunk* chunk = new Chunk;
   if(chunk != NULL) {
      chunk->NextChunk        = FirstChunk;
      chunk->AssociationID     = assocID;
      chunk->StreamID          = streamID;
      chunk->Length            = length;
      chunk->ProtoID           = protoID;
      chunk->Unordered         = unordered;
      chunk->Status            = status;
      chunk->ControlData       = controlData;
      chunk->ControlDataLength = controlDataLength;
      FirstChunk = chunk;

      signal();
      return(true);
   }

   cerr << "ERROR: ChunkArrivalQueue::pushChunk() - Out of memory!" << endl;
   return(false);
}


// ###### Get and remove chunk from queue ##################################
bool ChunkArrivalQueue::getChunk(unsigned int&            assocID,
                                 unsigned short&          streamID,
                                 unsigned int&            protoID,
                                 unsigned int&            length,
                                 unsigned int&            unordered,
                                 AssociationControlData*& controlData,
                                 unsigned int&            controlDataLength,
                                 AssociationStatus&       status)
{
   if(FirstChunk != NULL) {
      assocID           = FirstChunk->AssociationID;
      streamID          = FirstChunk->StreamID;
      protoID           = FirstChunk->ProtoID;
      length            = FirstChunk->Length;
      unordered         = FirstChunk->Unordered;
      status            = FirstChunk->Status;
      controlData       = FirstChunk->ControlData;
      controlDataLength = FirstChunk->ControlDataLength;

      Chunk* nextChunk = FirstChunk->NextChunk;
      if(LastChunk == FirstChunk) {
         LastChunk = nextChunk;
      }
      delete FirstChunk;
      FirstChunk = nextChunk;

      return(true);
   }
   return(false);
}


// ###### Wait for new chunk ###############################################
void ChunkArrivalQueue::waitForChunk()
{
   DataArriveCondition.wait();
}


// ###### Broadcast "new chunk" condition ##################################
void ChunkArrivalQueue::signal()
{
   DataArriveCondition.broadcast();
}


// ###### Flush all chunks #################################################
void ChunkArrivalQueue::flush()
{
   Chunk* chunk = FirstChunk;
   while(chunk != NULL) {
      Chunk* nextChunk = chunk->NextChunk;
      if(chunk->ControlData) {
         delete chunk->ControlData;
         chunk->ControlData = NULL;
      }
      delete chunk;
      chunk = nextChunk;
   }
   FirstChunk = NULL;
   LastChunk  = NULL;
}


// }
