


#include "sctpconnectionlesssocket.h"
#include "sctpsocketmaster.h"


// namespace Coral {



#define PRINT_NEW_ASSOCIATIONS
#define PRINT_SHUTDOWNS



// ###### Constructor #######################################################
SCTPConnectionlessSocket::SCTPConnectionlessSocket(const bool server)
   : SCTPSocket(true)
{
   if(server) {
      listen(10);
   }
}


// ###### Destructor ########################################################
SCTPConnectionlessSocket::~SCTPConnectionlessSocket()
{
}


// ###### Receive ###########################################################
int SCTPConnectionlessSocket::receiveFrom(char*              dataBuffer,
                                          size_t&            dataBufferSize,
                                          char*              controlBuffer,
                                          size_t&            controlBufferSize,
                                          int&               flags,
                                          unsigned short&    streamID,
                                          unsigned int&      protoID,
                                          InternetAddress**  addressArray,
                                          unsigned int&      addressEntries,
                                          AssociationStatus& status)
{
   // ====== Receive ========================================================
   unsigned int assocID = 0;
   const int result = internalReceive(
                         GlobalQueue,
                         dataBuffer, dataBufferSize,
                         controlBuffer, controlBufferSize,
                         flags,
                         assocID, streamID, protoID,
                         status);

   // ====== Extract sender's addresses =====================================
   if(((controlBufferSize > 0) || (dataBufferSize > 0)) && (addressArray != NULL)) {
      unsigned int i;
      for(i = 0;i < status.RemoteAddresses;i++) {
         if(i >= addressEntries) {
            break;
         }
         addressArray[i]->init(String((char*)&status.RemoteAddress[i]),status.RemotePort);
      }
      addressEntries = i;
   }

   // ====== Remove association, if error occured ===========================
   if((result == -ESHUTDOWN) || (result == -ECONNABORTED)) {
      SCTPAssociation* association = getAssociationForAssociationID(assocID);
      if(association != NULL) {
         SCTPSocketMaster::MasterInstance.lock();
         multimap<unsigned int, SCTPAssociation*>::iterator iterator =
            OutgoingAssociationList.find(assocID);
         if(iterator != OutgoingAssociationList.end()) {
            OutgoingAssociationList.erase(iterator);
         }
         SCTPSocketMaster::MasterInstance.unlock();
         delete association;
         return(0);
      }
   }

   return(result);
}


// ###### Send ##############################################################
int SCTPConnectionlessSocket::sendTo(const char*            buffer,
                                     const size_t           length,
                                     const int              flags,
                                     const unsigned short   streamID,
                                     const unsigned int     protoID,
                                     const InternetAddress& destinationAddress)
{
   InternetAddress destAddress(destinationAddress);
   destAddress.setPrintFormat(InternetAddress::PF_IPOnly|InternetAddress::PF_Address);

   SCTPSocketMaster::MasterInstance.lock();


   // ====== Check for already created association ==========================
   SCTPAssociation* association = NULL;
   multimap<unsigned int, SCTPAssociation*>::iterator iterator =
      OutgoingAssociationList.begin();
   while(iterator != OutgoingAssociationList.end()) {
      SCTP_Association_Status status;
      if(sctp_getAssocStatus(iterator->second->AssociationID,&status) == 0) {
         if( (!iterator->second->IsShuttingDown)                 &&
             (destAddress.getPort()          == status.destPort) &&
             (destAddress.getAddressString() == String((const char*)&status.primaryDestinationAddress)) ) {
            association = iterator->second;
            break;
         }
      }
      iterator++;
   }
   SCTPSocketMaster::MasterInstance.unlock();


   // ====== Create new association =========================================
   if(association == NULL) {
#ifdef PRINT_NEW_ASSOCIATIONS
      cout << "sendTo: New outgoing association to " << destAddress << "..." << endl;
#endif
      association = associate(1,destAddress);
      if(association != NULL) {
#ifdef PRINT_NEW_ASSOCIATIONS
         cout << "sendTo: New outgoing association successfully opened." << endl;
#endif
         OutgoingAssociationList.insert(pair<unsigned int, SCTPAssociation*>(association->getID(),association));
      }
#ifdef PRINT_NEW_ASSOCIATIONS
      else {
         cout << "sendTo: New outgoing association failed!" << endl;
      }
#endif
   }


   // ====== Send data ======================================================
   int result;
   if(association != NULL) {
      result = internalSend(buffer,length,flags,association->AssociationID,streamID,protoID);

      // ====== Remove association, if SHUTDOWN flag is set==================
      if(flags & MSG_SHUTDOWN) {
#ifdef PRINT_SHUTDOWNS
         cout << "sendTo: Shutdown of outgoing association to " << destAddress << "!" << endl;
#endif
         SCTPSocketMaster::MasterInstance.lock();
         multimap<unsigned int, SCTPAssociation*>::iterator iterator =
            OutgoingAssociationList.find(association->getID());
         if(iterator != OutgoingAssociationList.end()) {
            OutgoingAssociationList.erase(iterator);
         }
         SCTPSocketMaster::MasterInstance.unlock();
         delete association;
      }
   }
   else {
      result = -EIO;
   }

   return(result);
}


// }
