

#ifndef SCTPSOCKETWRAPPER_H
#define SCTPSOCKETWRAPPER_H


#ifndef IPPROTO_SCTP
    #define IPPROTO_SCTP 132
#endif


#ifdef USE_SCTP_WRAPPER

#include <sys/socket.h>
#include <fcntl.h>


extern "C" {

int td_socket(int domain, int type, int protocol);
int td_bind(int sockfd, struct sockaddr *my_addr, socklen_t addrlen);
int td_connect(int sockfd, const struct sockaddr *serv_addr,
               socklen_t addrlen);
int td_listen(int s, int backlog);
int td_accept(int s,  struct  sockaddr  *addr,  socklen_t *addrlen);
int td_shutdown(int s, int how);
int td_close(int fd);
int td_getsockname(int sockfd, struct sockaddr* name, socklen_t* namelen);
int td_getpeername(int sockfd, struct sockaddr* name, socklen_t* namelen);
int td_fcntl(int fd, int cmd = 0, struct flock * lock = NULL);
int td_ioctl(int d, int request, const void* argp);
int td_getsockopt(int sockfd, int level, int optname, void *optval, socklen_t *optlen);
int td_setsockopt(int sockfd, int level, int optname, const void *optval, socklen_t optlen);
int td_recv(int s, void *buf, size_t len, int flags);
int td_recvfrom(int  s,  void  *buf,  size_t len, int flags,
                struct sockaddr *from, socklen_t *fromlen);
int td_recvmsg(int s, struct msghdr *msg, int flags);
int td_send(int s, const void *msg, size_t len, int flags);
int td_sendto(int s, const void *msg, size_t len, int flags,
              const struct sockaddr *to, socklen_t tolen);
int td_sendmsg(int s, const struct msghdr *msg, int flags);
ssize_t td_read(int fd, void *buf, size_t count);
ssize_t td_write(int fd, const void *buf, size_t count);

}

#else

#define td_socket(a,b,c) ::socket(a,b,c)
#define td_bind(a,b,c) ::bind(a,b,c)
#define td_connect(a,b,c) ::connect(a,b,c)
#define td_listen(a,b) ::listen(a,b)
#define td_accept(a,b,c) ::accept(a,b,c)
#define td_shutdown(a,b) ::shutdown(a,b)
#define td_close(a) ::close(a)
#define td_getsockname(a,b,c) ::getsockname(a,b,c)
#define td_getpeername(a,b,c) ::getpeername(a,b,c)
#define td_fcntl(a,b,c) ::fcntl(a,b,c)
#define td_ioctl(a,b,c) ::ioctl(a,b,c)
#define td_getsockopt(a,b,c,d,e) ::getsockopt(a,b,c,d,e)
#define td_setsockopt(a,b,c,d,e) ::setsockopt(a,b,c,d,e)
#define td_recv(a,b,c,d) ::recv(a,b,c,d)
#define td_recvfrom(a,b,c,d,e,f) ::recvfrom(a,b,c,d,e,f)
#define td_recvmsg(a,b,c) ::recvmsg(a,b,c)
#define td_send(a,b,c,d) ::send(a,b,c,d)
#define td_sendto(a,b,c,d,e,f) ::sendto(a,b,c,d,e,f)
#define td_sendmsg(a,b,c) ::sendmsg(a,b,c)
#define td_read(a,b,c) ::read(a,b,c)
#define td_write(a,b,c) ::write(a,b,c)

#endif


#endif
