


#include "thread.h"


#include <sys/time.h>


// namespace Coral {


// ###### Static attributes #################################################
Synchronizable Thread::CriticalSectionLock;


// ###### Constructor #######################################################
Thread::Thread(cardinal flags)
   : Synchronizable()
{
   PThread = 0;
   Flags   = flags;
}


// ###### Destructor ########################################################
Thread::~Thread()
{
   stop();
}


// ###### Start the thread ##################################################
bool Thread::start()
{
   int result = -1;
   CriticalSectionLock.synchronized();
   synchronized();

   // Create new thread
   if(PThread == 0) {   
      // Create new thread
      result = pthread_create(&PThread,NULL,&go,(Thread*)this);
      if(result != 0) {
         cerr << "WARNING: Thread::start() - Unable to create pthread!" << endl;
      }
   }
   else {
      cerr << "WARNING: Thread::start() - Thread already started!" << endl;
   }
   
   unsynchronized();
   CriticalSectionLock.unsynchronized();
   return(result == 0);
}


// ###### Start function for pthread ########################################
void* Thread::go(void* argument)
{
   Thread* thisThread = (Thread*)argument;

   // Set parameters of the new thread
   int dummy;
   if(thisThread->Flags & ThreadCancelDeferred) {
      pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED,&dummy);
   }
   else {
      pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,&dummy);
   }

   // Call user's run() function
   thisThread->run();
   
   // Return with NULL result
   return((void*)NULL);
}


// ###### Stop the thread ###################################################
void Thread::stop()
{
  synchronized();
  if(running()) {
      pthread_cancel(PThread);
      join();

/*
      printf("resynchronize: owner=%08x count=%d kind=%d lstatus=%x lspinlock=%d\n",
             (cardinal)Mutex.__m_owner,Mutex.__m_count,Mutex.__m_kind,
             (cardinal)Mutex.__m_lock.__status,(cardinal)Mutex.__m_lock.__spinlock);
*/

      resynchronize();
      return;
   }
   unsynchronized();
}


// ###### Wait for end of thread ############################################
cardinal Thread::join()
{
   cardinal result;
   if(running()) {
      pthread_join(PThread,(void**)&result);
      PThread = 0;
   }
   return(result);
}


// ###### Sleep #############################################################
card64 Thread::delay(const card64 delayTimeout, const bool interruptable)
{
   struct timespec timeout;
   struct timespec remaining;
   timeout.tv_sec  = delayTimeout / 1000000;
   timeout.tv_nsec = 1000 * (delayTimeout % 1000000);

   int result = nanosleep(&timeout,&remaining);
   if(!interruptable) {
      while((result == -1) && (errno == EINTR)) {
         timeout.tv_sec  = remaining.tv_sec;
         timeout.tv_nsec = remaining.tv_nsec;
         result = nanosleep(&timeout,&remaining);
      }
   }
   else if((result == -1) && (errno == EINTR)) {
      return(((card64)remaining.tv_sec * 1000000) +
             ((card64)remaining.tv_nsec / 1000));
   }
   return(0);
}


// ###### Cancel thread #####################################################
void Thread::cancel()
{
   synchronized();
   if(running()) {
      pthread_cancel(PThread);
   }
   unsynchronized();
}


// ###### Test cancel #######################################################
void Thread::testCancel()
{
   pthread_testcancel();
}


// }
