


#include "unixaddress.h"


// namespace Coral {


// ###### Unix address constructor ##########################################
UnixAddress::UnixAddress()
{
   reset();
}


// ###### Unix address constructor ##########################################
UnixAddress::UnixAddress(const char* name)
{
   init(name);
}


// ###### Unix address constructor ######################################
UnixAddress::UnixAddress(const UnixAddress& address)
{
   init(address);
}


// ###### Unix address constructor ######################################
UnixAddress::~UnixAddress() 
{
}


// ###### Unix address constructor ######################################
UnixAddress::UnixAddress(sockaddr* address, const cardinal length)
{
   setSystemAddress(address,length);
}


// ###### Unix address constructor ######################################
void UnixAddress::init(const UnixAddress& address)
{
   init((char*)&address.Name);
}



// ###### Initialize ####################################################
void UnixAddress::init(const char* name)
{
   const cardinal length = strlen(name);
   if(length < MaxNameLength) {
      strcpy((char*)&Name,name);
   }
   else {
      Name[0] = 0x00;
      cerr << "WARNING: UnixAddress::init() - Name too long!" << endl;
   }
}


// ###### Reset #############################################################
void UnixAddress::reset()
{
   Name[0] = 0x00;
}


// ###### Check, if address is valid ########################################
bool UnixAddress::isValid() const
{
   return(!isNull());
}


// ###### Get address string ################################################
String UnixAddress::getAddressString() const
{
   if(Name[0] == 0x00) {
      return(String("(empty)"));
   }
   return(String((char*)&Name));
}


// ###### Get sockaddr structure from internet address ######################
cardinal UnixAddress::getSystemAddress(sockaddr* buffer,
                                           const cardinal length,
                                           const cardinal type) const
{
   switch(type) {
      case AF_UNIX: {
         sockaddr_un* address = (sockaddr_un*)buffer;
         if(sizeof(sockaddr_un) <= length) {
            address->sun_family = AF_UNIX;
            strncpy((char*)&address->sun_path,(char*)&Name,MaxNameLength);
            return(sizeof(sockaddr_un));
         }
         else {
            cerr << "WARNING: UnixAddress::getSystemUnixAddress() - "
                    "Buffer size too low for AF_UNIX!" << endl;
         }
        }
       break;
      default:
         cerr << "WARNING: UnixAddress::getSystemUnixAddress() - Unknown type "
              << type << "!" << endl;
       break;
   }
   return(0);
}


// ###### Initialize internet address from sockaddr structure ###############
bool UnixAddress::setSystemAddress(sockaddr* address, const cardinal length)
{
   sockaddr_un* unixAddress = (sockaddr_un*)address;
   switch(unixAddress->sun_family) {
      case AF_UNIX:
         strncpy((char*)&Name,(char*)&unixAddress->sun_path,MaxNameLength);
         return(true);
       break;
      default:
         reset();
        break;
   }
   return(false);
}


// }
